@@ -1,20 +1,20 @@
 # is a package installed?
-is.installed <- function(pkg) is.element(pkg, utils::installed.packages()[,1]) 
+is.installed <- function(pkg) is.element(pkg, utils::installed.packages()[,1])
 
 # generic FFT functions
 FFT <- function(X,inverse=FALSE)
 {
   if(is.null(ncol(X)) || is.na(ncol(X)))
-  { 
+  {
     if(!inverse) { X <- stats::fft(X) }
     else { X <- stats::fft(X,inverse=TRUE)/length(X) }
   }
   else
   {
-    if(!inverse) { X <- mvfft(X) }
+    if(!inverse) { X <- stats::mvfft(X) }
     else { X <- stats::mvfft(X,inverse=TRUE)/nrow(X) }
   }
-  
+
   return(X)
 }
 
@@ -22,7 +22,7 @@ FFT <- function(X,inverse=FALSE)
 FFTW <- function(X,inverse=FALSE)
 {
   if(is.null(ncol(X)) || is.na(ncol(X)))
-  { 
+  {
     if(!inverse) { X <- fftw::FFT(X) }
     else { X <- fftw::IFFT(X) }
   }
@@ -31,7 +31,7 @@ FFTW <- function(X,inverse=FALSE)
     if(!inverse) { X <- sapply(1:ncol(X),function(j){ fftw::FFT(X[,j]) }) }
     else { X <- sapply(1:ncol(X),function(j){ fftw::IFFT(X[,j]) }) }
   }
-  
+
   return(X)
 }
 
@@ -46,7 +46,7 @@ mclapply <- parallel::mclapply
 
   # don't try to fork if winblows
   if(.Platform$OS.type=="windows")
-  { 
+  {
     utils::assignInMyNamespace("detectCores", function(...) { 1 })
     utils::assignInMyNamespace("mclapply", function(X,FUN,mc.cores=1,...) { lapply(X,FUN,...) })
   }
@@ -156,18 +156,26 @@ cov.loglike <- function(hess,grad=rep(0,nrow(hess)))
   }
   # one of the curvatures is negative or close to negative
   # return something sensible just in case we are on a boundary and this makes sense
-  
+
   # normalize parameter scales by curvature or gradient (whatever is larger)
   V <- abs(diag(hess))
   V <- sqrt(V)
   V <- pmax(V,abs(grad))
+
+  # don't divide by zero
+  # TOL <- .Machine$double.eps * length(V)
+  TEST <- V==0
+  if(any(TEST)) { V[TEST] <- 1 }
+
   W <- V %o% V
-  
+
   grad <- grad/V
   hess <- hess/W
-  
+
   EIGEN <- eigen(hess)
   values <- EIGEN$values
+  if(any(values<=0)) { warning("MLE is near a boundary or optim failed.") }
+  values <- clamp(values,0,Inf)
   vectors <- EIGEN$vectors
 
   # transform gradient to hess' coordinate system
@@ -178,23 +186,31 @@ cov.loglike <- function(hess,grad=rep(0,nrow(hess)))
   for(i in 1:length(values))
   {
     DET <- values[i]+grad[i]^2
-    
+
     if(values[i]==0.0) # Wald limit of below
     { values[i] <- 1/(2*grad[i])^2 }
     else if(DET>=0.0) # Wald
-    { values[i] <- min(((c(1,-1)*sqrt(DET)-grad[i])/values[i])^2) }
+    { values[i] <- ((sqrt(DET)-grad[i])/values[i])^2 }
     else # minimum loglike? optim probably failed or hit a boundary
     {
-      warning("Likelihood has not been maximized. MLE could be near a boundary.")
-      # distance to worst parameter
-      values[i] <- (grad[i]/values[i])^2
-      # not sure what else to do...
+      # warning("MLE is near a boundary.")
+      # (parameter distance to worst parameter * 1/2 / loglike difference to worst parameter)^2
+      # values[i] <- 1/grad[i]^2
+      # pretty close to the other formula, so just using that
+      values[i] <- 1/(2*grad[i])^2
     }
   }
-  
-  COV <- vectors %*% diag(values,length(values)) %*% t(vectors)
+
+  COV <- array(0,dim(hess))
+  SUB <- values<Inf
+  if(any(SUB)) # separate out the finite part
+  { COV <- COV + Reduce("+",lapply((1:length(grad))[SUB],function(i){ values[i] * outer(vectors[,i]) })) }
+  SUB <- !SUB
+  if(any(SUB)) # toss out the off-diagonal NaNs
+  { COV <- COV + Reduce("+",lapply((1:length(grad))[SUB],function(i){ D <- diag(outer(vectors[,i])) ; D[D>0] <- Inf ; diag(D) })) }
+
   COV <- COV/W
-    
+
   return(COV)
 }
 
@@ -207,8 +223,12 @@ CI.lower <- Vectorize(function(k,Alpha){stats::qchisq(Alpha/2,k,lower.tail=TRUE)
 chisq.ci <- function(MLE,COV=NULL,alpha=0.05,DOF=2*MLE^2/COV)
 {
   # try to do something reasonable on failure cases
-  if(MLE==0)
+  if(DOF==0)
+  { CI <- c(0,MLE,Inf) }
+  else if(MLE==0)
   { CI <- c(0,0,0) }
+  else if(MLE==Inf)
+  { CI <- c(Inf,Inf,Inf) }
   else if(!is.null(COV) && COV<0) # try an exponential distribution?
   {
     warning("VAR[Area] = ",COV," < 0")
@@ -219,7 +239,7 @@ chisq.ci <- function(MLE,COV=NULL,alpha=0.05,DOF=2*MLE^2/COV)
   else     # regular estimate
   {
     CI <- MLE * c(CI.lower(DOF,alpha),1,CI.upper(DOF,alpha))
-    
+
     # qchisq upper.tail is too small when DOF<<1
     # probably an R bug that no regular use of chi-square/gamma would come across
     if(is.null(COV)) { COV <- 2*MLE^2/DOF }
@@ -227,7 +247,7 @@ chisq.ci <- function(MLE,COV=NULL,alpha=0.05,DOF=2*MLE^2/COV)
     UPPER <- norm.ci(MLE,COV,alpha=alpha)[3]
     if(CI[3]<UPPER) { CI[3] <- UPPER }
   }
-    
+
   names(CI) <- c("low","ML","high")
   return(CI)
 }
@@ -237,10 +257,10 @@ norm.ci <- function(MLE,COV,alpha=0.05)
 {
   # z-values for low, ML, high estimates
   z <- stats::qnorm(1-alpha/2)*c(-1,0,1)
-  
+
   # normal ci
   CI <- MLE + z*sqrt(COV)
-  
+
   names(CI) <- c("low","ML","high")
   return(CI)
 }
@@ -252,9 +272,9 @@ lognorm.ci <- function(MLE,COV,alpha=0.05)
   COV <- COV/MLE^2
   # log transform of point estimate
   MLE <- log(MLE)
-  
+
   CI <- norm.ci(MLE,COV,alpha=alpha)
-  
+
   # transform back
   CI <- exp(CI)
 
@@ -282,12 +302,12 @@ pad <- function(vec,size,padding=0,side="right")
   # this is now the pad length instead of total length
   size <- size - length(vec)
   padding <- array(padding,size)
-  
+
   if(side=="right"||side=="r")
   { vec <- c(vec,padding) }
   else if(side=="left"||side=="l")
   { vec <- c(padding,vec) }
-  
+
   return(vec)
 }
 
@@ -298,17 +318,17 @@ rpad <- function(mat,size,padding=0,side="right")
   COL <- ncol(mat)
   padding <- array(padding,c(size,COL))
   colnames(padding) <- colnames(mat)
-    
+
   if(side=="right"||side=="r")
   { mat <- rbind(mat,padding) }
   else if(side=="left" || side=="l")
   { mat <- rbind(padding,mat) }
-  
+
   return(mat)
 }
 
 #remove rows and columns by name
 rm.name <- function(object,name)
 {
-  object[!rownames(object) %in% name,!colnames(object) %in% name] 
+  object[!rownames(object) %in% name,!colnames(object) %in% name]
 }
